#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import unittest
import sys


def start():
    argv = ['emoji-unicode', 'discover']

    if len(sys.argv) > 1:
        argv = sys.argv

    unittest.main(module=None, argv=argv)


if __name__ == '__main__':
    start()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import unittest
import logging
import os
import json
import io

from emoji_unicode import replace, normalize, Emoji
from emoji_unicode.utils import code_point_to_unicode, unicode_to_code_point
from emoji_unicode import data_parser


logging.disable(logging.CRITICAL)

DIR = os.path.dirname(__file__)
FIXTURES = os.path.join(DIR, 'fixtures')
EMOJI_PRETTY_JSON = None


def _get_emoji_pretty():
    global EMOJI_PRETTY_JSON

    if EMOJI_PRETTY_JSON is not None:
        return EMOJI_PRETTY_JSON

    with io.open(os.path.join(FIXTURES, 'emoji_pretty.json'), encoding='utf-8') as fh:
        EMOJI_PRETTY_JSON = fh.read()

    return EMOJI_PRETTY_JSON


def get_emoji_pretty():
    return json.loads(_get_emoji_pretty())


def code_points_to_unicode(code_points):
    return ''.join(
        code_point_to_unicode(p)
        for p in code_points.split('-')
    )


def get_emojis(include_skin_variations=True, include_variations=True):
    # todo: include variations (emoji + emo_variation), android doesn't use them, check iOS
    emojis = []

    for e in get_emoji_pretty():
        emojis.append({
            'unicode': code_points_to_unicode(e['unified']),
            'code_point': e['unified'],
            'short_name': e['short_name']
        })

        if include_skin_variations:
            emojis.extend(
                {
                    'unicode': code_points_to_unicode(point),
                    'code_point': point,
                    'short_name': e['short_name']
                }
                for point in e.get('skin_variations', {}).keys()
            )

        if include_variations:
            emojis.extend(
                {
                    'unicode': code_points_to_unicode(point),
                    'code_point': point,
                    'short_name': e['short_name']
                }
                for point in e.get('variations', [])
            )

    return emojis


def get_emojis_unicode(**kw):
    return [e['unicode'] for e in get_emojis(**kw)]


class MetaTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_code_points_to_unicode(self):
        self.assertEqual(
            code_points_to_unicode('1F58B-1F58B-1F58B'),
            '\U0001f58b\U0001f58b\U0001f58b'
        )

    def test_get_emojis(self):
        self.assertEqual(len(get_emojis()), 1736)
        self.assertEqual(len(get_emojis(include_skin_variations=False)), 1416)
        self.assertEqual(len(get_emojis(include_variations=False)), 1619)

    def test_get_emojis_unicode(self):
        self.assertEqual(len(get_emojis_unicode()), 1736)
        self.assertEqual(len(get_emojis_unicode(include_skin_variations=False)), 1416)
        self.assertEqual(len(get_emojis(include_variations=False)), 1619)


class UtilsTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_code_point_to_unicode(self):
        self.assertEqual(
            code_point_to_unicode('1F58B'),
            '\U0001f58b'
        )

    def test_unicode_to_code_point(self):
        self.assertEqual(
            unicode_to_code_point('\U0001f58b'),
            '1F58B'.lower()
        )


class ModelEmojiTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_unicode(self):
        emoji = Emoji(unicode='foo')
        self.assertEqual(emoji.unicode, 'foo')

    def test_code_points(self):
        emoji = Emoji(unicode='\U0001f58b\U0001f58b\U0001f58b')
        self.assertEqual(emoji.code_points, '1F58B-1F58B-1F58B'.lower())


class ParserTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_replace(self):
        """
        It should replace all emojis
        """
        emojis = get_emojis()

        # With no spaces will fail due to fitzpatrick tone being a modifier and also a emoji
        txt = ' '.join(get_emojis_unicode())
        txt_code_points = ' '.join(normalize(e['code_point']) for e in emojis)
        res = replace(txt, lambda emoji: emoji.code_points)
        self.assertEqual(res, txt_code_points)

    def test_replace_with_no_fitz(self):
        """
        It should replace no-spaced emojis, excluding fitzpatrick tone emojis
        """
        emojis = get_emojis()
        txt = ''.join(
            e['unicode']
            for e in emojis
            if 'skin-tone' not in e['short_name']
        )
        txt_code_points = ''.join(
            normalize(e['code_point'])
            for e in emojis
            if 'skin-tone' not in e['short_name']
        )
        res = replace(txt, lambda emoji: emoji.code_points)
        self.assertEqual(res, txt_code_points)

    def test_replace_remove(self):
        txt = ''.join(get_emojis_unicode())
        res = replace(txt, lambda emoji: '')
        self.assertEqual(res, '')

    def test_replace_digits(self):
        """
        It should not match single digits
        """
        txt = '#*0123456789'
        res = replace(txt, lambda emoji: '')
        self.assertEqual(res, txt)

    def test_replace_text_variations(self):
        """
        It should not match emojis with text variation
        """
        txt = '\u203C\uFE0E'
        res = replace(txt, lambda emoji: '')
        self.assertEqual(res, txt)

    def test_normalize(self):
        self.assertEqual(normalize('00A900'), 'a900')

    def test_normalize_variations(self):
        self.assertEqual(normalize('00A9-FE0F-200D-F00'), 'a9-f00')

    def test_normalize_separator(self):
        self.assertEqual(normalize('00A9_FE0F_200D_F00', separator='_'), 'a9_f00')


class DataParserTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_parse(self):
        res = set(data_parser.parse())
        self.assertTrue('\u00A9' in res)
        self.assertTrue('\u2194-\u2199' in res)  # range

    def test_read_template(self):
        template = data_parser.read_template()
        self.assertTrue('{{code_points}}' in template)
        self.assertTrue('RE_PATTERN_TEMPLATE' in template)

    def test_render_template(self):
        code_points = data_parser.parse()
        template = data_parser.read_template()
        rendered_template = data_parser.render_template(template, code_points)
        self.assertTrue('{{code_points}}' not in rendered_template)
        self.assertTrue('RE_PATTERN_TEMPLATE' in rendered_template)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function
import re
import functools
import timeit

from emoji_unicode import data_parser
from emoji_unicode.parser import replace, RE_PATTERN_TEMPLATE


if __name__ == '__main__':
    data_parser.generate_pattern_file()

    txt = '⛽'
    txt *= 10000
    print('emoji.replace()')
    print('text len: {}'.format(len(txt)))
    print(timeit.timeit(functools.partial(replace, txt, lambda x: ''), number=1))

    print('\nre.sub() (raw match)')
    print('text len: {}'.format(len(txt)))
    pattern = re.compile(RE_PATTERN_TEMPLATE)
    print(timeit.timeit(functools.partial(re.sub, pattern, lambda x: '', txt), number=1))

    txt = 'a'
    txt *= 10000
    print('\nText with no emojis')
    print('emoji.replace()')
    print('text len: {}'.format(len(txt)))
    print(timeit.timeit(functools.partial(replace, txt, lambda x: ''), number=1))



# -*- coding: utf-8 -*-

from __future__ import unicode_literals


def code_point_to_unicode(code_point):
    try:
        return unichr(int(code_point, 16))
    except NameError:
        return chr(int(code_point, 16))


def unicode_to_code_point(uni_char):
    return format(ord(uni_char), 'x').lower()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals


CODE_POINTS = '{{code_points}}'  # Template string

TXT_VARIATION = '\uFE0E'
EMO_VARIATION = '\uFE0F'
FITZ_MODIFIER = '\U0001F3FB-\U0001F3FF'
KC_MODIFIER = '\u20E3'
ZWJ = '\u200D'
FLAGS = '\U0001F1E6-\U0001F1FF'
KEY_CAPS = '0-9\*#'

RE_PATTERN_TEMPLATE = (
    r'(?P<emoji>'
        r'(?:'
            r'(?:[%(key_caps)s](?:%(emo_variation)s)?%(kc_modifier)s)'
            r'|'
            r'(?:[%(flags)s]){2}'
            r'|'
            r'(?:[%(emojis)s])(?!%(txt_variation)s)'
        r')'
        r'(?:'
            r'(?:(?:%(emo_variation)s)?(?:[%(fitz_modifier)s]))'  # fitzpatrick modifier
            r'|'
            r'(?:(?:%(emo_variation)s)?(?:[%(zwj)s])(?:.)){1,4}'  # Multi glyphs (up to 4)
            r'|'
            r'(?:%(emo_variation)s)'  # Emoji variation
        r')?'
    r')'
) % {
    'emojis': CODE_POINTS,
    'txt_variation': TXT_VARIATION,
    'emo_variation': EMO_VARIATION,
    'fitz_modifier': FITZ_MODIFIER,
    'zwj': ZWJ,
    'flags': FLAGS,
    'kc_modifier': KC_MODIFIER,
    'key_caps': KEY_CAPS
}  # noqa



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import re

from .models import Emoji, JOINER_CHARS
from .utils import code_point_to_unicode
from .pattern import RE_PATTERN_TEMPLATE

PATTERN = re.compile(RE_PATTERN_TEMPLATE)


def replace(txt, callback):
    """
    Replace all unicode emojis in a given text

    :param str txt: Text source to be parsed
    :param callable callback: A callable that should accept\
    a instance of :py:class:`Emoji` and\
    return a str to replace the match
    :return: Parsed text with all the emojis\
    replaced by the callback result
    :rtype: str
    """
    def func(m):
        return callback(Emoji(unicode=m.group('emoji')))

    return re.sub(PATTERN, func, txt)


def normalize(code_points, separator='-'):
    """
    Normalize a code point removing joiner\
    characters such as emoji variations,\
    Zero Width Joiner and leading zeros

    :param str code_points: code points separated by an hyphen\
    or the separator param value
    :param str separator: The separator used to split the code points,\
    process them and merge them back
    :return: Code points with no joiner chars,\
    leading zeros, and lower cased.
    :rtype: str
    """
    return separator.join(
        c.lstrip('0').lower()
        for c in code_points.split(separator)
        if code_point_to_unicode(c) not in JOINER_CHARS
    )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from .pattern import RE_PATTERN_TEMPLATE
from .parser import replace, normalize
from .models import Emoji

__version__ = '0.1'

__all__ = [
    'RE_PATTERN_TEMPLATE',
    'replace',
    'normalize',
    'Emoji'
]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from .utils import unicode_to_code_point
from .pattern import EMO_VARIATION, ZWJ

JOINER_CHARS = {EMO_VARIATION, ZWJ}


class Emoji(object):
    """
    Emoji is used in the process of :py:func:`.replace` unicode emojis in a text

    :param str unicode: Unicode emoji
    """
    def __init__(self, unicode):
        self.unicode = unicode

    @property
    def code_points(self):
        """
        Code points representing the unicode emoji,\
        the result is normalized as by :py:func:`.normalize`

        :return: Code points representing the emoji,\
        with no joiner chars and lower cased, ie: 1f3c3-1f3fc
        :rtype: str
        """
        return '-'.join(
            unicode_to_code_point(u)
            for u in self.unicode
            if u not in JOINER_CHARS
        )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

# This script is base on:
# http://www.unicode.org/Public/emoji/latest/emoji-data.txt
# and (Unicode 8.0 emojis)
# http://unicode.org/reports/tr51/

# Invisible variation selectors:
# U+FE0E for a text presentation
# U+FE0F for an emoji presentation
#
# U+200D ZERO WIDTH JOINER (ZWJ) can be used between the elements
# of a sequence of characters to indicate that a single glyph
# should be presented if available.
#
# EMOJI MODIFIER FITZPATRICK
# U+1F3FB..U+1F3FF
#
# Variation selector (U+FE0F) may or may not be between Fitz modifier and the emoji
#
# Regional indicators:
# 1F1E6..1F1FF
#
# Mark modifiers (ie: square-digits)
# 20E0 enclosing_circle_backlash
# 20E3 enclosing_keycap
#
# emoji sequence:
# (emoji_modifier_base | emoji_base_variation_sequence) | emoji_modifier
#
# emoji core sequence:
# emoji_modifier_base | emoji_base_variation_sequence | emoji_modifier | emoji_flag_sequence

import os
import io

from .utils import code_point_to_unicode


DIR = os.path.dirname(__file__)


def escape_unicode(txt):
    return txt \
        .encode('unicode_escape') \
        .replace(b'\\', b'\\\\') \
        .decode('unicode_escape')


def _parse(line):
    code_point = line.split(';', 1)[0]
    return '-'.join(
        code_point_to_unicode(c)
        for c in code_point.strip().split('..')
    )


def parse():
    with io.open(os.path.join(DIR, 'emoji-data.txt'), mode='r', encoding='utf-8') as fh:
        return [
            _parse(line)
            for line in fh.readlines()
            if not line.startswith('#')
        ]


def read_template():
    with io.open(os.path.join(DIR, 'pattern_template.py'), mode='r', encoding='utf-8') as fh:
        return fh.read()


def render_template(template, code_points):
    code_points = escape_unicode(''.join(code_points))
    return template.replace('{{code_points}}', code_points)


def write_pattern_file(template_rendered):
    with io.open(os.path.join(DIR, 'pattern.py'), mode='w', encoding='utf-8') as fh:
        fh.write(template_rendered)


def generate_pattern_file():
    code_points = parse()
    template = read_template()
    template_rendered = render_template(template, code_points)
    write_pattern_file(template_rendered)


